@@ -21,6 +21,8 @@
 #  include <config.h>
 #endif
 
+#define PY_SIZE_T_CLEAN
+
 /* macro to simplify conditionals based on ORBit version */
 #include <orbit/orbit-config.h>
 #define ORBIT_VERSION_CHECK(x,y,z) \
@@ -179,6 +181,9 @@ pyorbit_servant_generic_skel_func(PortableServer_ServantBase *servant,
     PyObject *args = NULL, *ret = NULL;
     gint n_args, n_rets, i, argpos, retpos;
     gboolean has_ret;
+    PyGILState_STATE state;
+
+    state = pyorbit_gil_state_ensure();
 
     pyservant = SERVANT_TO_PYSERVANT(servant);
     imethod = (ORBit_IMethod *)impl;
@@ -253,7 +258,7 @@ pyorbit_servant_generic_skel_func(PortableServer_ServantBase *servant,
 	/* wrap a tuple round the return value ... */
 	ret = Py_BuildValue("(N)", ret);
     } else if (n_rets + (has_ret ? 1 : 0) != PySequence_Length(ret)) {
-	g_message("%s: return sequence length is wrong (expected %d, got %d)",
+	g_warning("%s: return sequence length is wrong (expected %d, got %d)",
 		  imethod->name, n_rets + (has_ret ? 1 : 0),
 		  PySequence_Length(ret));
 	CORBA_exception_set_system(ev, ex_CORBA_DATA_CONVERSION,
@@ -268,7 +273,7 @@ pyorbit_servant_generic_skel_func(PortableServer_ServantBase *servant,
 	item = PySequence_GetItem(ret, retpos++);
 	if (!item) {
 	    PyErr_Clear();
-	    g_message("%s: couldn't get return val", imethod->name);
+	    g_warning("%s: couldn't get return val", imethod->name);
 	    CORBA_exception_set_system(ev, ex_CORBA_DATA_CONVERSION,
 				       CORBA_COMPLETED_MAYBE);
 	    goto cleanup;
@@ -294,7 +299,7 @@ pyorbit_servant_generic_skel_func(PortableServer_ServantBase *servant,
 	}
 	if (!pyorbit_marshal_any(&any, item)) {
 	    Py_DECREF(item);
-	    g_message("%s: could not marshal return", imethod->name);
+	    g_warning("%s: could not marshal return", imethod->name);
 	    CORBA_exception_set_system(ev, ex_CORBA_DATA_CONVERSION,
 				       CORBA_COMPLETED_MAYBE);
 	    goto cleanup;
@@ -316,7 +321,7 @@ pyorbit_servant_generic_skel_func(PortableServer_ServantBase *servant,
 	item = PySequence_GetItem(ret, retpos++);
 	if (!item) {
 	    PyErr_Clear();
-	    g_message("%s: could not get arg from tuple", imethod->name);
+	    g_warning("%s: could not get arg from tuple", imethod->name);
 	    CORBA_exception_set_system(ev, ex_CORBA_DATA_CONVERSION,
 				       CORBA_COMPLETED_MAYBE);
 	    goto cleanup;
@@ -341,7 +346,7 @@ pyorbit_servant_generic_skel_func(PortableServer_ServantBase *servant,
 
 	if (!pyorbit_marshal_any(&any, item)) {
 	    Py_DECREF(item);
-	    g_message("%s: could not marshal arg", imethod->name);
+	    g_warning("%s: could not marshal arg", imethod->name);
 	    CORBA_exception_set_system(ev, ex_CORBA_DATA_CONVERSION,
 				       CORBA_COMPLETED_MAYBE);
 	    goto cleanup;
@@ -353,6 +358,7 @@ pyorbit_servant_generic_skel_func(PortableServer_ServantBase *servant,
     Py_XDECREF(method);
     Py_XDECREF(args);
     Py_XDECREF(ret);
+    pyorbit_gil_state_release(state);
 }
 
 static ORBitSmallSkeleton
@@ -376,16 +382,44 @@ impl_finder_func(PortableServer_ServantBase *servant,
     return pyorbit_servant_generic_skel_func;
 }
 
+  /* caller must CORBA_Object_release the POA */
+static PortableServer_POA
+_pyorbit_servant_get_poa(PyPortableServer_Servant *self)
+{
+    PyObject *pypoa;
+    PortableServer_POA poa;
+
+    pypoa = PyObject_CallMethod((PyObject *)self, "_default_POA", NULL);
+    if (!pypoa)
+	return CORBA_OBJECT_NIL;
+    if (!PyObject_TypeCheck(pypoa, &PyPortableServer_POA_Type)) {
+	Py_DECREF(pypoa);
+	PyErr_SetString(PyExc_TypeError, "could not lookup default POA");
+	return CORBA_OBJECT_NIL;
+    }
+    poa = (PortableServer_POA) CORBA_Object_duplicate(((PyCORBA_Object *)pypoa)->objref, NULL);
+    Py_DECREF(pypoa);
+    return poa;
+}
+
 static void
 pyorbit_servant_dealloc(PyPortableServer_Servant *self)
 {
     PortableServer_ServantBase *servant;
 
-    Py_XDECREF(self->this);
-    Py_XDECREF(self->delegate);
-    self->this = NULL;
     servant = PYSERVANT_TO_SERVANT(self);
+
+    if (self->activator_poa) {
+        PortableServer_ObjectId *id;
+          /* deactivate the object */
+        id = PortableServer_POA_servant_to_id(self->activator_poa, servant, NULL);
+        PortableServer_POA_deactivate_object(self->activator_poa, id, NULL);
+        CORBA_free(id);
+        CORBA_Object_release((CORBA_Object) self->activator_poa, NULL);
+    }
     PortableServer_ServantBase__fini(servant, NULL);
+    Py_CLEAR(self->this);
+    Py_CLEAR(self->delegate);
 
     if (self->ob_type->tp_free)
 	self->ob_type->tp_free((PyObject *)self);
@@ -425,6 +459,7 @@ pyorbit_servant_new(PyTypeObject *type, PyObject *args, PyObject *kwargs)
 
     servant = PYSERVANT_TO_SERVANT(self);
     servant->vepv = info->vepv;
+    ORBit_classinfo_register(&info->class_info);
     ORBIT_SERVANT_SET_CLASSINFO(servant, &info->class_info);
 
     CORBA_exception_init(&ev);
@@ -470,8 +505,6 @@ static PyObject *
 pyorbit_servant__this(PyPortableServer_Servant *self)
 {
     CORBA_Environment ev;
-    PyObject *pypoa;
-    PortableServer_POA poa;
     PortableServer_ServantBase *servant;
     PortableServer_ObjectId *objid;
     CORBA_Object objref;
@@ -480,31 +513,22 @@ pyorbit_servant__this(PyPortableServer_Servant *self)
 	Py_INCREF(self->this);
 	return self->this;
     }
-
-    /* this is meant to call self._default_POA */
-    pypoa = PyObject_CallMethod((PyObject *)self, "_default_POA", NULL);
-    if (!pypoa)
-	return NULL;
-    if (!PyObject_TypeCheck(pypoa, &PyPortableServer_POA_Type)) {
-	Py_DECREF(pypoa);
-	PyErr_SetString(PyExc_TypeError, "could not lookup default POA");
-	return NULL;
-    }
-    poa = (PortableServer_POA)((PyCORBA_Object *)pypoa)->objref;
-
+    
+    g_assert(!self->activator_poa);
+    self->activator_poa = _pyorbit_servant_get_poa(self);
+    if (G_UNLIKELY(self->activator_poa == CORBA_OBJECT_NIL))
+        return NULL;
     servant = PYSERVANT_TO_SERVANT(self);
 
     CORBA_exception_init(&ev);
-    objid = PortableServer_POA_activate_object(poa, servant, &ev);
+    objid = PortableServer_POA_activate_object(self->activator_poa, servant, &ev);
     CORBA_free(objid);
     if (pyorbit_check_ex(&ev)) {
-	Py_DECREF(pypoa);
     	return NULL;
     }
     
     CORBA_exception_init(&ev);
-    objref = PortableServer_POA_servant_to_reference(poa, servant, &ev);
-    Py_DECREF(pypoa);
+    objref = PortableServer_POA_servant_to_reference(self->activator_poa, servant, &ev);
     if (pyorbit_check_ex(&ev))
     	return NULL;
 
