@@ -21,6 +21,9 @@
 #  include <config.h>
 #endif
 #include <string.h>
+
+#define PY_SIZE_T_CLEAN
+
 #include "pyorbit-private.h"
 
 /* repo_id -> CORBA_TypeCode hash */
@@ -214,7 +217,7 @@ generate_enum_stub(CORBA_TypeCode tc)
     PyObject *container;
     PyObject *stub, *values;
     gchar *pyname;
-    gint i;
+    Py_ssize_t i;
 
     container = _pyorbit_get_container(tc->repo_id, FALSE);
     if (!container)
