@@ -30,14 +30,14 @@ SHORT_INOUT_OUT  = 0x5678
 SHORT_OUT        = 0x7812
 SHORT_RETN       = -21829 # 0xAABB
 
-SEQ_STRING_IN        = ( 'in1', 'in2', 'in3', 'in4' )
-SEQ_STRING_INOUT_IN  = ('inout1', 'inout2', 'inout3', 'inout4' )
-SEQ_STRING_INOUT_OUT = ( 'inout21', 'inout22', 'inout23', 'inout24' )
-SEQ_STRING_OUT       = ( 'out1', 'out2', 'out3', 'out4' )
-SEQ_STRING_RETN      = ( 'retn1', 'retn2', 'retn3', 'retn4' )
+SEQ_STRING_IN        = [ 'in1', 'in2', 'in3', 'in4' ]
+SEQ_STRING_INOUT_IN  = ['inout1', 'inout2', 'inout3', 'inout4' ]
+SEQ_STRING_INOUT_OUT = [ 'inout21', 'inout22', 'inout23', 'inout24' ]
+SEQ_STRING_OUT       = [ 'out1', 'out2', 'out3', 'out4' ]
+SEQ_STRING_RETN      = [ 'retn1', 'retn2', 'retn3', 'retn4' ]
 
-SEQ_LONG_IN          = ( LONG_IN, LONG_INOUT_IN, 15, 7 )
-SEQ_LONG_INOUT_IN    = ( LONG_INOUT_OUT, LONG_OUT, 7, 15 )
-SEQ_LONG_INOUT_OUT   = ( LONG_OUT, LONG_RETN, 8, 9 )
-SEQ_LONG_OUT         = ( LONG_INOUT_IN, LONG_INOUT_OUT, 15, 7 )
-SEQ_LONG_RETN        = ( LONG_RETN, LONG_IN, 2, 3 )
+SEQ_LONG_IN          = [ LONG_IN, LONG_INOUT_IN, 15, 7 ]
+SEQ_LONG_INOUT_IN    = [ LONG_INOUT_OUT, LONG_OUT, 7, 15 ]
+SEQ_LONG_INOUT_OUT   = [ LONG_OUT, LONG_RETN, 8, 9 ]
+SEQ_LONG_OUT         = [ LONG_INOUT_IN, LONG_INOUT_OUT, 15, 7 ]
+SEQ_LONG_RETN        = [ LONG_RETN, LONG_IN, 2, 3 ]
